function [samples, probs, info] = momMatchScenLMI(meanVec, R, kappa, s)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NAME: Moment Matching Scenario Generator (using LMI toolbox)
% AUTHOR: Luka Jalen
% DATE: 12.04.2010
%
% DESCRIPTION:
% This function calculates the samples from a multivariate symmetric 
% distribution matching first three moments exactly and also matches
% marginal fourth moments. The exact description of the algorithm can be 
% found in
%
% [1] P. Date, R.S. Mamon and L. Jalen, A new moment matching algorithm 
% for sampling from partially specified symmetric distributions, 
% Operations Research Letters 36 (2008), no. 6, 669-672.
%
%
% ARGUMENTS:
% The function expects 4 arguments:
%   meanVec : a vector of mean values 
%   R : covariance matrix 
%   kappa : a vector of fourth moments
%   s : desired number of samples (the actual number of sample points
%   returned is 2*n*s + 1 where n is the length of meanVec)
%
%
% RETURN VALUES:
%   samples : a matrix of sample points
%   probs : a vector of corresponding probability weights 
%   info : a struct with fields containing additional information such as
%           time requirement for the calculation, absolute and maximum 
%           error in matching the fourth moment and kurtosis of the
%           returned sample points. Fields are:
%               info.kurtosis
%               info.epsilon
%               info.diff
%               info.time
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Some housekeeping left to do
n = length(meanVec);
L = chol(R);

%Initialise the LMI description and define a new matrix variable
setlmis([])
x = lmivar(1, repmat([1 1], s + 1, 1));

% Define the LMI terms or in other words introduce the constraints set in
% [1] under equations (2) - (4)

% The first term as specified in [1] in equation (2)
epsMat = zeros(1, s + 1);
epsMat(1) = 1;

psiMat = ones(1, s + 1);
psiMat(1) = 0;

for ii = 1 : n
    lmiterm([-(2 + ii) 1 1 1],  epsMat, epsMat');
    lmiterm([-(2 + ii) 1 2 1],  sum(L(ii, :).^4)*psiMat, (1/(2*s^2))*psiMat');
    lmiterm([-(2 + ii) 1 2 0], -kappa(ii));
    lmiterm([-(2 + ii) 2 2 1],  epsMat, epsMat');
end

% The second term as specified in [1] in equation (3)
A3 = eye(s + 1);
A3 = A3(2 : s + 1, :);
lmiterm([-1 1 1 1], A3, A3');

% And the last term as specified in [1] in equation (4)
lmiterm([-2 1 1 0], 1);
lmiterm([-2 1 2 0], zeros(1, s));
lmiterm([-2 2 2 1], A3, A3' * 1/(2*n));

% Convert the description above to the internal representation of the LMI
% system and minimise the objective with LMI constraints
myLMI = getlmis;

tic
[copt,xopt] = mincx(myLMI,epsMat);
time = toc;

% Calculate the probability weights
p = zeros(1, s+1);
p(1:s) = 1./xopt(2 : s + 1);
p(s + 1) = 1 - 2* n * sum(p);

% And finally, generate sample points and corresponding probability weights
samples = zeros(n, 2*n*s + 1);
probs = zeros(1, 2*n*s + 1);

for ii = 1 : s 
    for jj = 1 : n
        samples(:, (ii - 1)* n + jj) = meanVec - ...
            (L(:, jj)/sqrt(2 * s * p(ii)));
        samples(:, 2*n*s+2 -((ii - 1)* n + jj)) = meanVec + ...
            (L(:, jj)/sqrt(2 * s * p(ii)));
        
        % same for probs
        probs((ii - 1)* n + jj) = p(ii);
        probs(2*n*s+2 -((ii - 1)* n + jj)) = p(ii);
    end
end

% Simply add the mean, and we're done:
samples(:, n*s + 1) = meanVec;
probs(n*s + 1) = p(s+1);

% The only thing left to be done is gather the additional information and
% pack it in 'info'
calcKurt = zeros(n, 1);
for ii = 1 : n
    currMean = samples(ii, :)*probs';
    calcKurt(ii) = ((samples(ii, :) - currMean).^4)*probs';
end
diff = abs(calcKurt - kappa');
info.kurtosis = calcKurt;
info.epsilon = xopt(1);
info.diff = diff;
info.time = time;
